#!/usr/bin/local/python
# -*- coding: utf-8 -*-

import sys
import os
import threading
import wx
import wx_utils
from wx_utils import XRC, XRCID, XRCCTRL
from called_step import OpenEditor

try:
	import win32gui
except:
	win32gui = None

import main_icon
import pickle

append_path = os.path.split(wx_utils.executable())[0]
sys.path.append(append_path)
import custom
from custom.default import get_stc, get_wildcards
from custom.default import get_wildcard_index, get_description
import	wx.stc	as	stc

conf_filename = os.path.splitext(wx_utils.executable())[0] + '.conf'

class FileDropTarget(wx.FileDropTarget):
	def __init__(self, parent):
		wx.FileDropTarget.__init__(self)
		self.parent = parent
	def OnDropFiles(self, x, y, filenames):
		for file in filenames:
			self.parent.OpenFile(file)

class MainFrame(wx.Frame):
	u"""MainFrame class deffinition.
	"""
	binder = wx_utils.bind_manager()
	TIMER_ID = 0
	
	def __init__(self, parent=None):
		pre = wx.PreFrame()
		XRC().LoadOnFrame(pre, parent, 'MainFrame')
		self.PostCreate(pre)
		self.SetMenuBar(XRC().LoadMenuBar('MainMenubar'))
		self.binder.bindall(self)
		self.SetIcon(main_icon.getIcon())
		self.multitab = XRCCTRL(self, 'MultiTab')
		self.SetDropTarget(FileDropTarget(self))
		self.load()
		self.hoge=False
		self.timer = wx.Timer(self)
		self.lock = threading.Lock()

	def load(self):
		try:
			self.config = pickle.load(file(conf_filename))
		except:
			self.config = {}
		self.SetPosition(self.config.get('window.position', (-1, -1)))
		self.SetSize(self.config.get('window.size', (480, 320)))

	def save(self):
		self.config['window.position'] = self.GetPositionTuple()
		self.config['window.size'] = self.GetSizeTuple()
		pickle.dump(self.config, file(conf_filename,'w'))

	@binder(wx.EVT_MIDDLE_DOWN, control='MultiTab')
	def OnTabMiddleDown(self, event):
		index = self.multitab.HitTest(event.GetPositionTuple())
		if index!=wx.NOT_FOUND:
			self.multitab.SetSelection(index[0])

	@binder(wx.EVT_MIDDLE_UP, control='MultiTab')
	def OnTabMiddleUp(self, event):
		index = self.multitab.HitTest(event.GetPositionTuple())
		if index!=wx.NOT_FOUND and self.multitab.GetSelection()==index[0]:
			self.OnClose(event)

	@binder(wx.EVT_NOTEBOOK_PAGE_CHANGED, control='MultiTab')
	def OnPageChanged(self, event):
		event.Skip()
		editor = self.multitab.GetPage(event.GetSelection())
		if editor:
			method, fname, encoding = editor.GetName().split('|')
			sb = self.GetStatusBar()
			sb.SetStatusText(encoding, 2)
			sb.SetStatusText(get_description(os.path.splitext(fname)[1]),1)

	@binder(wx.EVT_ACTIVATE)
	def OnActivate(self, event):
		event.Skip()
		if not event.GetActive():
			self.save()

	@binder(wx.EVT_CLOSE)
	def OnWindowClose(self, event):
		event.Skip()
		self.OnAllClose(None)

	@binder(wx.EVT_MENU, id='wxID_EXIT')
	def OnQuit(self, event):
		self.OnAllClose(None)
		self.Close()

	@binder(wx.EVT_MENU, id='wxID_ABOUT')
	def OnAbout(self, event):
		wx.MessageBox(
			self.GetTitle(),
			u'情報', wx.OK | wx.ICON_INFORMATION )

	@binder(wx.EVT_MENU, id='wxID_NEW')
	def OnNew(self, event):
		#self.OpenFile()
		self.OpenFile("test.py")
		self.OpenFile("test2.py")
		"""
		editor = self.multitab.GetCurrentPage()
		if editor:
			line_len = editor.GetLineCount()
			for i in range(1,line_len+1):
				#j = line_len - i + 1
				j = i
				print j
				if j % 2 == 0:
					editor.SetCalledLine("test.py", j,"line")
				else:
					editor.SetCalledLine("test.py", j, "return")
				print editor.GetEndStyled()
		#"""

	@binder(wx.EVT_MENU, id='wxID_OPEN')
	def OnOpen(self, event):
		wildcards = '|'.join(get_wildcards())
		dlg = wx.FileDialog(
						self, message=u"開くファイルの選択",
						defaultDir=os.getcwd(), 
						defaultFile="", wildcard=wildcards,
						style=wx.OPEN | wx.MULTIPLE | wx.FILE_MUST_EXIST)
		res = dlg.ShowModal()
		if res==wx.ID_OK:
			for fname in dlg.GetFilenames():
				self.OpenFile(fname)

	@binder(wx.EVT_MENU, id='wxID_SAVE')
	def OnSave(self, event):
		self.SaveFile(self.multitab.GetSelection())

	@binder(wx.EVT_MENU, id='wxID_SAVE_AS')
	def OnSaveAs(self, event):
		self.SaveAsFile(self.multitab.GetSelection())

	@binder(wx.EVT_MENU, id='wxID_ALL_SAVE')
	def OnAllSave(self, event):
		for i in range(self.multitab.GetPageCount()):
			self.SaveFile(i)

	@binder(wx.EVT_MENU, id='wxID_CLOSE')
	def OnClose(self, event):
		index = self.multitab.GetSelection()
		editor = self.multitab.GetPage(index)
		method, fname, encoding = editor.GetName().split('|')
		if editor.GetModify():
			while not self.SaveFile(index):
				if method=='new':
					break
		self.multitab.DeletePage(index)

	@binder(wx.EVT_MENU, id='wxID_ALL_CLOSE')
	def OnAllClose(self, event):
		for i in reversed(range(self.multitab.GetPageCount())):
			self.multitab.SetSelection(i)
			self.OnClose(None)

	def SaveAsFile(self, index):
		editor = self.multitab.GetPage(index)
		if editor:
			wildcards = '|'.join(get_wildcards())
			method, fname, encoding = editor.GetName().split('|')
			dlg = wx.FileDialog(
							self, message=u'名前をつけて保存',
							defaultDir=os.getcwd(), 
							defaultFile=fname, wildcard=wildcards,
							style=wx.SAVE | wx.OVERWRITE_PROMPT)
			res = dlg.ShowModal()
			if res == wx.ID_OK:
				fname = dlg.GetPath()
				file(fname, 'w').write(editor.GetText().encode(encoding))
				editor.SetName('|'.join(['open', fname, encoding]))
				editor.SetSavePoint()
				self.multitab.SetPageText(index, os.path.split(fname)[1])
				return True
		return False

	def SaveFile(self, index):
		editor = self.multitab.GetPage(index)
		method, fname, encoding = editor.GetName().split('|')
		if method=='new':
			return self.SaveAsFile(index)
		else:
			file(fname, 'w').write(editor.GetText().encode(encoding))
			editor.SetSavePoint()
			title = self.multitab.GetPageText(index).rstrip(' *')
			self.multitab.SetPageText(index, title)
			return True

	def OpenFile(self, fname=''):
		for i in range(self.multitab.GetPageCount()):
			editor = self.multitab.GetPage(i)
			method, exists_fname, encoding = editor.GetName().split('|')
			if fname==exists_fname:
				self.multitab.SetSelection(i)
				return
		method = 'open'
		encoding = sys.getdefaultencoding()
		if fname:
			fname = os.path.abspath(fname)
			title = os.path.split(fname)[1]
		else:
			base = 'New'
			index = 1
			titles = []
			for i in range(self.multitab.GetPageCount()):
				titles.append(self.multitab.GetPageText(i))
			title = '%(base)s%(index)d' % vars()
			while title in titles:
				index += 1
				title = '%(base)s%(index)d' % vars()
			method = 'new'
			fname = os.path.abspath(title)
		name , ext = os.path.splitext(fname)
		klass = get_stc(ext)
		editor = klass(self.multitab, -1)
		editor.StatusBar = self.GetStatusBar()
		editor.Bind(wx.stc.EVT_STC_MODIFIED, self.OnDocumentChange)
		
		if method=='open':
			text = file(fname).read()
			contents = text
			for enc in ['shift-jis', 'utf-8', 'utf-16']:
				try:
					contents = unicode(text, enc)
					encoding = enc
					break
				except UnicodeDecodeError:
					pass
			editor.SetText(contents)
		editor.SetDropTarget(FileDropTarget(self))
		editor.SetName('|'.join([method, fname, encoding]))
		editor.EmptyUndoBuffer()
		self.multitab.AddPage(editor, title, True)
			

	@binder(wx.EVT_MENU, id='wxID_COPY')
	def Copy(self,event):
		editor = self.multitab.GetCurrentPage()
		if editor:
			editor.Copy()

	@binder(wx.EVT_MENU, id='wxID_CUT')
	def Cut(self,event):
		editor = self.multitab.GetCurrentPage()
		if editor:
			editor.Cut()

	@binder(wx.EVT_MENU, id='wxID_PASTE')
	def Paste(self,event):
		editor = self.multitab.GetCurrentPage()
		if editor:
			editor.Paste()

	@binder(wx.EVT_MENU, id='wxID_UNDO')
	def Undo(self,event):
		editor = self.multitab.GetCurrentPage()
		if editor:
			editor.Undo()

	@binder(wx.EVT_MENU, id='wxID_REDO')
	def Redo(self,event):
		editor = self.multitab.GetCurrentPage()
		if editor:
			editor.Redo()

	@binder(wx.EVT_MENU, id='wxID_DELETE')
	def Delete(self,event):
		editor = self.multitab.GetCurrentPage()
		if editor:
			editor.Clear()

	@binder(wx.EVT_MENU, id='wxID_SELECT_ALL')
	def SelectAll(self,event):
		editor = self.multitab.GetCurrentPage()
		if editor:
			editor.SelectAll()
	
	@binder(wx.EVT_MENU, id="wxID_STYLE_CLEAR")
	def StyleClear(self, event):
		editor = self.multitab.GetCurrentPage()
		if editor:
			#editor.ResetStyle()
			self.AllStyleClear()
	
	@binder(wx.EVT_MENU, id="wxID_DEBUG_REAL_RUN")
	def RealTimeRun(self, event):
		""" 通過した行の色を徐々に薄くしていく """
		self.DebugRun(event, False)
	
	@binder(wx.EVT_MENU, id="wxID_DEBUG_HIGH_RUN")
	def HighlightRun(self, event):
		""" 通過した行の色をそのままにしておく """
		self.DebugRun(event, True)
	
	def DebugRun(self, event, highlight):
		editor = self.multitab.GetCurrentPage()
		if editor:
			print "DebugRun"
			method, exists_fname, encoding = editor.GetName().split('|')
			print method, exists_fname, encoding
			sys.path[0] = os.path.dirname(exists_fname)
			self.open_editors = {}
			self.AllStyleClear()
			self.open_editors = self.GetOpenEditors()
			if highlight:
				for value in self.open_editors.values():
					value.SetStartColor((0,255,0))
					value.SetEndColor((0,255,0))
			
			#editor.ResetStyle()
			from debug import MyBdb, Debugger
			self.debug = Debugger(exists_fname, self.lock, self.open_editors)
			self.debug.setDaemon(False)
			self.debug.start()
			
			self.timer.Start(100)
	
	def AllStyleClear(self):
		for i in range(self.multitab.GetPageCount()):
			editor = self.multitab.GetPage(i)
			editor.StyleClearAll()
			editor.StyleSetDefault()
	
	def GetOpenEditors(self):
		editors = {}
		for i in range(self.multitab.GetPageCount()):
			editor = self.multitab.GetPage(i)
			method, filename, encoding = editor.GetName().split('|')
			filename = filename.lower().encode('sjis')
			if filename not in self.open_editors:
				editors[filename] = OpenEditor(editor)
		return editors
	
	def OnDebugFinish(self):
		self.timer.Stop()
		#editor = self.multitab.GetCurrentPage()
		#editor.PrintStyle()

	def ColoringLine(self):
		self.lock.acquire()
		open_editors = self.open_editors
		called_list = self.debug.called_line
		#print "ColoringLine"
		# 各行をエディタごとに色づけ
		for file in open_editors.keys():
			editor = open_editors[file].editor
			called = open_editors[file].called_step
			called.Regist(called_list.get(file, []))
			for i in range(called.save_time):
				line_list = called.GetLines(i)
				#print i, line_list
				color = called.GetColor(i)
				style = called.GetStyleNum(i)
				for line in line_list:
					editor.SetLineColor(line, color, style)
			cleared_line = called.GetLastCleared()
			#print cleared_line
			for line in cleared_line:
				editor.SetLineColor(line, called.end_color, stc.STC_STYLE_DEFAULT)
				
		"""
		for line in self.debug.called_line:
			editor.SetCalledLine(*line)
		"""
		self.debug.ClearCalledLine()
		self.lock.release()
	
	@binder(wx.EVT_TIMER)
	def OnTimer(self, event):
		if not self.open_editors:
			return
		self.GetOpenEditors()
		self.ColoringLine()
		if not self.debug.isAlive():
			self.OnDebugFinish()
	
	def OnDocumentChange(self, event):
		event.Skip()
		for i in range(self.multitab.GetPageCount()):
			if self.multitab.GetPage(i).GetModify():
				title = self.multitab.GetPageText(i).rstrip(' *')
				self.multitab.SetPageText(i, title + ' *')

# startup application.
if __name__=='__main__':
	app = wx.App(False)
	wx_utils.XrcInit()
	frame = MainFrame()
	app.SetTopWindow(frame)
	opend = False
	for arg in sys.argv[1:]:
		if os.path.exists(arg):
			frame.OpenFile(arg)
			opend = True
	if not opend:
		frame.OnNew(None)
	frame.Show()
	app.MainLoop()
